@@ -41,6 +41,7 @@
 #include <sys/ioctl.h>
 #include <sys/mman.h>
 #include <sys/stat.h>
+#include <sys/io.h>
 #include <linux/vt.h>
 #include <linux/fb.h>
 #include <asm/page.h>
@@ -57,6 +58,15 @@ static int tfbm_select_visual(  TFrameBufferMemory* p,
 				struct fb_fix_screeninfo* fbfs);
 
 static TFrameBufferCapability sFBCapabilityList[] = {
+#if (defined(JFB_2BPP) && defined(JFB_PACKED) && defined(JFB_PSEUDOCOLOR))
+	{
+		2, FB_TYPE_PACKED_PIXELS, FB_VISUAL_PSEUDOCOLOR,
+		tfbm_fill_rect_2bpp_packed,
+		tfbm_overlay_2bpp_packed,
+		tfbm_clear_all_2bpp_packed,
+		tfbm_reverse_2bpp_packed
+	},
+#endif
 #if (defined(JFB_8BPP) && defined(JFB_PACKED) && defined(JFB_PSEUDOCOLOR))
 	{
 		8, FB_TYPE_PACKED_PIXELS, FB_VISUAL_PSEUDOCOLOR,
@@ -180,13 +190,11 @@ static struct fb_cmap	ocmap = { 0, 256, ored, ogreen, oblue, otrans};
 
 static int tvisual = 0;
 
-#ifdef FORCE_8BIT
-static int force_8bit = 0;
-static int odepth;
-#endif /* FORCE_8BIT */
-
 float fbgamma = 1.7;
 
+static struct fb_var_screeninfo ovar;
+static TBool modified_var_screen_info = FALSE;
+
 /*---------------------------------------------------------------------------*/
 static u_int trueColor32Table[16];
 static u_short trueColor16Table[16];
@@ -225,14 +233,12 @@ static void tfbm_get_var_screen_info(int fh, struct fb_var_screeninfo *var)
 	}
 }
 
-#if FORCE_8BIT
 static void tfbm_set_var_screen_info(int fh, struct fb_var_screeninfo *var)
 {
 	if (ioctl(fh, FBIOPUT_VSCREENINFO, var)) {
 		print_strerror_and_exit("ioctl FBIOPUT_VSCREENINFO");
 	}
 }
-#endif
 
 static void tfbm_get_fix_screen_info(int fh, struct fb_fix_screeninfo *fix)
 {
@@ -262,6 +268,37 @@ static void tfbm_pan_display(int fh, struct fb_var_screeninfo *var)
 	}
 }
 
+#if defined(JFB_ENABLE_DIMMER) && defined(FBIOBLANK)
+int tfbm_set_blank(int fh, int blank)
+{
+      int fd;
+      struct vt_stat vstat;
+      if (-1 == (fd = open("/dev/tty0", O_RDWR, 0))) {
+              die("open /dev/tty0: %s\n", strerror(errno));
+      }
+      if (-1 == ioctl(fd, VT_GETSTATE, &vstat)) {
+              die("ioctl VT_GETSTATE");
+      }
+      if (vstat.v_active != gFramebuffer.my_vt)
+              return(0);         /* Not foreground console */
+
+      if (ioctl(fh, FBIOBLANK, blank)) {
+              print_strerror_and_exit("ioctl FBIOBLANK");
+      }
+      return (1);     // Did it.
+}
+#endif
+
+#if defined(JFB_2BPP)
+static void simple_2bpp_palette(void)
+{
+        red[0] = green[0] = blue[0] = 0x0000;
+        red[1] = green[1] = blue[1] = 0x5555;
+        red[2] = green[2] = blue[2] = 0xaaaa;
+        red[3] = green[3] = blue[3] = 0xffff;
+}
+#endif
+
 static void linear_palette(int bit)
 {
 	int i;
@@ -293,6 +330,8 @@ static void dither_palette(int r, int g, int b)
 	}
 }
 
+extern void set_most_left(__u32 bpp, struct fb_bitfield a_pixel_field);
+
 static void tfbm_initcolors(TFrameBufferMemory* p,
 			    struct fb_var_screeninfo* fbvs,
 			    struct fb_fix_screeninfo* fbfs)
@@ -306,6 +345,12 @@ static void tfbm_initcolors(TFrameBufferMemory* p,
 	switch (fbvs->bits_per_pixel) {
 	case 1:
 		break;
+#if defined(JFB_2BPP)
+	case 2:
+		simple_2bpp_palette();
+		set_most_left(2, fbvs->red);
+		break;
+#endif
 	case 4:
 		break;
 	case 8:
@@ -419,7 +464,11 @@ void tfbm_init(TFrameBufferMemory* p)
 			die("ioctl VT_GETSTATE");
 		}
 		close(fd);
+#ifdef JFB_ENABLE_DIMMER
+		p->my_vt = c2m.console = vstat.v_active;
+#else
 		c2m.console = vstat.v_active;
+#endif
 		if (-1 == (fd = open("/dev/fb0", O_RDWR,0))) {
 			util_free(fbdn);
 			die("open /dev/fb0: %s\n", strerror(errno));
@@ -452,12 +501,16 @@ void tfbm_open(TFrameBufferMemory* p)
 	free(fbdn);
 
 	tfbm_get_var_screen_info(p->fh, &fb_var);
+	if (fb_var.yres_virtual != fb_var.yres) {
+		memcpy((void*)&ovar,(void*)&fb_var,sizeof(ovar));
+		fb_var.yres_virtual = fb_var.yres;
+		fb_var.yoffset = 0;
+		fb_var.activate = FB_ACTIVATE_NOW;
+		modified_var_screen_info = TRUE;
+		tfbm_set_var_screen_info(p->fh, &fb_var);
+	}
 	tfbm_get_fix_screen_info(p->fh, &fb_fix);
 
-#ifdef DEBUG
-	tfbm_show_screeninfo(p, &fb_var, &fb_fix);
-#endif
-
 	if (fb_fix.visual == FB_VISUAL_DIRECTCOLOR ||
 	    fb_fix.visual == FB_VISUAL_PSEUDOCOLOR) {
 		tfbm_get_cmap(p->fh, &ocmap);
@@ -466,26 +519,28 @@ void tfbm_open(TFrameBufferMemory* p)
 
 #ifdef FORCE_8BIT
 	if(fb_fix.visual == FB_VISUAL_DIRECTCOLOR) {
-		struct fb_var_screeninfo var;
-		memcpy((void*)&var,(void*)&fb_var,sizeof(var));
-		tfbm_get_cmap(p->fh, &ocmap);
-		odepth = fb_var.bits_per_pixel;
+		if (modified_var_screen_info == FALSE) {
+			memcpy((void*)&ovar,(void*)&fb_var,sizeof(ovar));
+			modified_var_screen_info = TRUE;
+		}
 		fb_var.bits_per_pixel = 8;
 		fb_var.vmode |= FB_VMODE_CONUPDATE;
-		force_8bit = 1;
+		fb_var.activate = FB_ACTIVATE_NOW;
 		tfbm_set_var_screen_info(p->fh, &fb_var);
 		tfbm_get_fix_screen_info(p->fh, &fb_fix);
-		tfbm_get_var_screen_info(p->fh, &fb_var);
 		if (fb_fix.type != FB_TYPE_PACKED_PIXELS ||
 				fb_var.bits_per_pixel != 8) {
-			var.vmode |= FB_VMODE_CONUPDATE;
-			tfbm_set_var_screen_info(p->fh, &var);
+			ovar.vmode |= FB_VMODE_CONUPDATE;
+			ovar.activate = FB_ACTIVATE_NOW;
+			tfbm_set_var_screen_info(p->fh, &ovar);
 			die("fb does not support 8bpp packed format\n");
 		}
 	}
 #endif /* FORCE_8BIT */
 
-
+#ifdef DEBUG
+	tfbm_show_screeninfo(p, &fb_var, &fb_fix);
+#endif
 	tvisual = tfbm_select_visual(p, &fb_var, &fb_fix);
 	if (tvisual < 0) {
 		die("Oops: Unknown frame buffer ???\n");
@@ -501,30 +556,31 @@ void tfbm_open(TFrameBufferMemory* p)
 	p->bytePerLine = fb_fix.line_length;
 
 	p->soff = (u_long)(fb_fix.smem_start) & (~PAGE_MASK);
-	p->slen = fb_fix.smem_len + p->soff;
-	p->smem = (u_char*)mmap(NULL, fb_fix.smem_len, PROT_READ|PROT_WRITE,
-				MAP_SHARED, p->fh, (off_t)0) + p->soff;
-	if ((long)(p->smem - p->soff) == -1) {
+	p->slen = (fb_fix.smem_len + p->soff + ~PAGE_MASK) & PAGE_MASK;
+	p->smem = (u_char*)mmap(NULL, p->slen, PROT_READ|PROT_WRITE,
+				MAP_SHARED, p->fh, (off_t)0);
+	if ((long)p->smem == -1) {
 		die("cannot mmap(smem)");
 	}
+	p->smem = (char *)p->smem + p->soff;
 
 	p->moff = (u_long)(fb_fix.mmio_start) & (~PAGE_MASK);
-	p->mlen = fb_fix.mmio_len + p->moff;
-	p->mmio = (u_char*)mmap(NULL, fb_fix.mmio_len, PROT_READ|PROT_WRITE,
-				MAP_SHARED, p->fh, fb_fix.smem_len) + p->moff;
-
-#ifdef DEBUG
-	print_message("mmap ; %d - %p\n", p->slen, p->smem);
-	print_message("mmio ; %d - %p\n", p->mlen, p->mmio);
-#endif
-
-	if ((long)(p->mmio - p->moff) == -1) {
+	p->mlen = (fb_fix.mmio_len + p->moff + ~PAGE_MASK) & PAGE_MASK;
+	p->mmio = (u_char*)mmap(NULL, p->mlen, PROT_READ|PROT_WRITE,
+				MAP_SHARED, p->fh, p->slen);
+	if ((long)p->mmio == -1) {
 #ifdef JFB_MMIO_CHECK 
 		die("cannot mmap(mmio)");
 #else
 		print_message("cannot mmap(mmio) : %s\n", strerror(errno));
 #endif
 	}
+	p->mmio = (char *)p->mmio + p->moff;
+
+#ifdef DEBUG
+	print_message("mmap ; %d - %p\n", p->slen, p->smem);
+	print_message("mmio ; %d - %p\n", p->mlen, p->mmio);
+#endif
 
 	/* move viewport to upper left corner */
 	if (fb_var.xoffset != 0 || fb_var.yoffset != 0) {
@@ -552,26 +608,21 @@ void tfbm_close(TFrameBufferMemory* p)
 	if (p->fh == -1) {
 		return;
 	}
-	if ((long)(p->smem - p->soff) != -1) {
-		munmap((caddr_t)((u_long)(p->smem - p->soff)), p->slen);
+	if ((long)p->smem != -1) {
+		munmap((caddr_t)((u_long)p->smem & PAGE_MASK), p->slen);
 	}
-	if ((long)(p->mmio - p->moff) != -1) {
-		munmap((caddr_t)((u_long)(p->mmio - p->moff)), p->mlen);
+	if ((long)p->mmio != -1) {
+		munmap((caddr_t)((u_long)p->mmio & PAGE_MASK), p->mlen);
 	}
-#ifdef FORCE_8BIT
-	if(force_8bit) {
-		struct fb_var_screeninfo fb_var;
-		tfbm_get_var_screen_info(p->fh, &fb_var);
-		fb_var.bits_per_pixel = odepth;
-		fb_var.vmode |= FB_VMODE_CONUPDATE;
-		tfbm_set_var_screen_info(p->fh, &fb_var);
-		tfbm_put_cmap(p->fh, &ocmap);
-	} else
-#endif /* FORCE_8BIT */
 	if (cmapSaved == TRUE) {
 		tfbm_put_cmap(p->fh, &ocmap);
 		cmapSaved = FALSE;
 	}
+	if (modified_var_screen_info == TRUE) {
+		ovar.activate = FB_ACTIVATE_NOW;
+		tfbm_set_var_screen_info(p->fh, &ovar);
+		modified_var_screen_info = FALSE;
+	}
 	close(p->fh);
 }
 
@@ -580,13 +631,25 @@ int tfbm_select_visual( TFrameBufferMemory* p,
 			struct fb_fix_screeninfo* fbfs)
 {
 	int ret;
+	int static_color = 0;
+
+	if(fbfs->visual == FB_VISUAL_STATIC_PSEUDOCOLOR){
+	    fbfs->visual = FB_VISUAL_PSEUDOCOLOR;
+	    static_color = 1;
+	}
 	for(ret = 0 ; sFBCapabilityList[ret].bitsPerPixel != 0 ; ret++) {
 		if (sFBCapabilityList[ret].fbType == fbfs->type &&
 		    sFBCapabilityList[ret].fbVisual == fbfs->visual &&
 		    sFBCapabilityList[ret].bitsPerPixel==fbvs->bits_per_pixel){
+			if(static_color){
+			    fbfs->visual = FB_VISUAL_STATIC_PSEUDOCOLOR;
+			}
 			return ret;
 		}
 	}
+	if(static_color){
+		fbfs->visual = FB_VISUAL_STATIC_PSEUDOCOLOR;
+	}
 	return -1;
 }
 
