@@ -28,10 +28,210 @@
 #include <stdio.h>
 #include <string.h>
 #include <endian.h>
+#include <linux/fb.h>
 
 #include <fbdpsp.h>
 #include <fbcommon.h>
 
+#ifndef min
+#  define min(a, b)   (((a)>(b))?(b):(a))
+#endif
+
+#ifdef JFB_2BPP
+/* 2bpp */
+
+static u_char mask_2bpp_msb_left[] = {
+	0xc0, 0x30, 0x0c, 0x03
+};
+static u_char mask_2bpp_msb_right[] = {
+	0x03, 0x0c, 0x30, 0xc0
+};
+static u_char* mask_2bpp = mask_2bpp_msb_left;
+
+void set_most_left(__u32 bpp, struct fb_bitfield red){
+	if(red.offset != 0) return;
+	if(red.length != bpp) return;
+	/*
+		This probing mechanism is known to work on:
+		NEC MobileGear(DOS model), NEC MobileGear-II, 
+		Psion s5. (but not work on Psion 5mx...)
+		Call for more precise way...
+	*/
+	if(red.msb_right){
+		mask_2bpp = mask_2bpp_msb_right;
+	}
+}
+
+static u_char color_map_2bpp[] = {
+	0x00,   // 0 (black)
+	0x55, 0x55, 0x55, 0x55, 0x55, 0x55,
+	0xff,   // 7 (normal white)
+	0x55,   // 8 (reverse underlined background)
+	0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa,
+	0xff    // 15 (hilight white)
+};
+
+static inline u_char tfbm_select_2_color(u_int color)
+{
+#ifdef JFB_REVERSEVIDEO
+	return (~ color_map_2bpp[color & 0xf]);
+#else
+	return (color_map_2bpp[color & 0xf]);
+#endif
+}
+
+static inline void set_pixel_2bpp_packed(
+	u_char *p, u_int x, u_int icol)
+{
+	// p points byte to be written
+	u_int b = x / 4;        // Byte offset
+	p[b] = ((p[b] & ~mask_2bpp[x & 3]) | (mask_2bpp[x & 3] & icol));
+}
+
+static inline void set_byte_2bpp_packed(
+	u_char *bp, u_int icol)
+{
+	*bp = icol;
+}
+
+static inline void reverse_pixel_2bpp_packed(
+	u_char *p, u_int x, u_int icol)
+{
+	*p = (*p & ~mask_2bpp[x & 3]) | ((*p ^ icol) & mask_2bpp[x & 3]);
+}
+
+void tfbm_fill_rect_2bpp_packed(
+	TFrameBufferMemory* p,
+	u_int sx, u_int sy, u_int lx, u_int ly, u_int color)
+{
+	u_int y;
+	u_int icol = tfbm_select_2_color(color);
+	for (y = sy ; y < sy+ly ; y++) {
+		u_char *line = p->smem + y * p->bytePerLine;
+		u_char *bp = line + (sx / 4);   // byte position
+		u_int xx = sx % 4;
+		u_int pixels = lx;
+		if (xx != 0) {
+			// Fill until byte boundary
+			while (xx % 4 != 0 && pixels != 0) {
+				set_pixel_2bpp_packed(bp, xx++, icol);
+				pixels--;
+			}
+			bp++;
+		}
+		while (pixels >= 4) {
+			// Fill with byte
+			set_byte_2bpp_packed(bp, icol);
+			pixels -= 4;
+			bp++;
+		}
+		xx = 0;
+		while (pixels != 0) {
+			// Fill rest pixels
+			set_pixel_2bpp_packed(bp, xx++, icol);
+			pixels--;
+		}
+	}
+}
+
+void tfbm_clear_all_2bpp_packed(
+	TFrameBufferMemory* p, u_int color)
+{
+	u_int icol = tfbm_select_2_color(color);
+	memset(p->smem, icol, p->slen);
+}
+
+void tfbm_overlay_2bpp_packed(
+	TFrameBufferMemory* p,
+	u_int xd, u_int yd,
+	const u_char* ps, u_int lx, u_int ly, u_int gap, u_int color)
+{
+	u_int y;
+	u_char* wp;
+	const u_char* tps;
+	u_int i;
+	u_int sb;
+	u_int icol = tfbm_select_2_color(color);
+
+	for (y = yd ; y < yd+ly ; y++) {
+		u_int xx = xd % 4;
+		tps = ps;
+		wp = p->smem + y * p->bytePerLine + (xd / 4);
+		for (i = lx ; i >= 8 ; i -= 8) {
+			sb = *tps++;
+			if (sb & 0x80)
+				set_pixel_2bpp_packed(wp, xx, icol);
+			xx++;
+			if (sb & 0x40)
+				set_pixel_2bpp_packed(wp, xx, icol);
+			xx++;
+			if (sb & 0x20)
+				set_pixel_2bpp_packed(wp, xx, icol);
+			xx++;
+			if (sb & 0x10)
+				set_pixel_2bpp_packed(wp, xx, icol);
+			xx++;
+			if (sb & 0x08)
+				set_pixel_2bpp_packed(wp, xx, icol);
+			xx++;
+			if (sb & 0x04)
+				set_pixel_2bpp_packed(wp, xx, icol);
+			xx++;
+			if (sb & 0x02)
+				set_pixel_2bpp_packed(wp, xx, icol);
+			xx++;
+			if (sb & 0x01)
+				set_pixel_2bpp_packed(wp, xx, icol);
+			xx++;
+		}
+		if (i) {
+			sb = *tps++;
+			switch (i) {
+			case 7:
+				if (sb & 0x02)
+					set_pixel_2bpp_packed(wp, xx + 6, icol);
+			case 6:
+				if (sb & 0x04)
+					set_pixel_2bpp_packed(wp, xx + 5, icol);
+			case 5:
+				if (sb & 0x08)
+					set_pixel_2bpp_packed(wp, xx + 4, icol);
+			case 4:
+				if (sb & 0x10)
+					set_pixel_2bpp_packed(wp, xx + 3, icol);
+			case 3:
+				if (sb & 0x20)
+					set_pixel_2bpp_packed(wp, xx + 2, icol);
+			case 2:
+				if (sb & 0x40)
+					set_pixel_2bpp_packed(wp, xx + 1, icol);
+			case 1:
+				if (sb & 0x80)
+					set_pixel_2bpp_packed(wp, xx + 0, icol);
+			}
+		}
+		ps += gap;
+	}
+}
+
+void tfbm_reverse_2bpp_packed(
+	TFrameBufferMemory* p,
+	u_int sx, u_int sy, u_int lx, u_int ly, u_int color)
+{
+	u_int y;
+	u_int x;
+	u_int icol = tfbm_select_2_color(color);
+	for (y = sy ; y < min(sy+ly, p->height) ; y++) {
+		unsigned char *line = p->smem + y * p->bytePerLine;
+		for (x = sx ; x < sx + lx ; x++) {
+			unsigned char *bp = line + (x / 4);     // byte position
+			reverse_pixel_2bpp_packed(bp, x, icol);
+		}
+	}
+}
+
+#endif /* JFB_2BPP */
+
 
 #ifdef JFB_8BPP
 /* 8 bpp */
@@ -65,7 +265,7 @@ void tfbm_fill_rect_8bpp_packed(
 	u_int sx, u_int sy, u_int lx, u_int ly, u_int color)
 {
 	u_int y;
-	for (y = sy ; y < sy+ly ; y++) {
+	for (y = sy ; y < min(sy+ly, p->height) ; y++) {
 #if 0
  		memset(p->smem + y * p->bytePerLine + sx, color, lx);
 #else
@@ -177,7 +377,7 @@ void tfbm_reverse_8bpp_packed(
 	T_DWORD *wp;
 
 	icol = color; icol |= icol << 8; icol |= icol << 16;
-	for (y = sy ; y < sy+ly ; y++) {
+	for (y = sy ; y < min(sy+ly, p->height) ; y++) {
 		wp = (T_DWORD*)(p->smem + y * p->bytePerLine + (sx & ~3u));
 		bm = 0xF >> (sx & 3);
 		x = lx + (sx & 3);
@@ -194,7 +394,7 @@ void tfbm_reverse_8bpp_packed(
 #else
 	u_int y;
 	u_int x;
-	for (y = sy ; y < sy+ly ; y++) {
+	for (y = sy ; y < min(sy+ly, p->height) ; y++) {
 		for (x = sx ; x < sx+lx ; x++) {
 			p->smem[y * p->bytePerLine + x] ^= color;
 		}
@@ -215,7 +415,7 @@ void tfbm_fill_rect_15bpp_packed(
 	u_short icol;
 
 	icol = tfbm_select_16_color(color);
-	for (y = sy ; y < sy+ly ; y++) {
+	for (y = sy ; y < min(sy+ly, p->height) ; y++) {
 		d=(u_short*)(p->smem + y * p->bytePerLine + sx * 2);
 		for(x=0;x<lx;x++){
 			*d = icol;
@@ -291,7 +491,7 @@ void tfbm_reverse_15bpp_packed(
 	u_short icol;
 
 	icol = tfbm_select_16_color(color);
-	for (y = sy ; y < sy+ly ; y++) {
+	for (y = sy ; y < min(sy+ly, p->height) ; y++) {
 		d = (u_short*)(p->smem + y * p->bytePerLine + sx * 2);
 		for (x = 0 ; x < lx ; x++) {
 			d[x] ^= icol;
@@ -311,7 +511,7 @@ void tfbm_fill_rect_16bpp_packed(
 	u_short icol;
 
 	icol = tfbm_select_16_color(color);
-	for (y = sy ; y < sy+ly ; y++) {
+	for (y = sy ; y < min(sy+ly, p->height) ; y++) {
 		d=(u_short*)(p->smem + y * p->bytePerLine + sx * 2);
 		for(x=0;x<lx;x++){
 			*d = icol;
@@ -377,6 +577,10 @@ void tfbm_overlay_16bpp_packed(
 	}
 }
 
+#ifndef min
+#  define min(a, b)    (((a) > (b))?(b):(a))
+#endif
+
 void tfbm_reverse_16bpp_packed(
 	TFrameBufferMemory* p,
 	u_int sx, u_int sy, u_int lx, u_int ly, u_int color)
@@ -386,7 +590,7 @@ void tfbm_reverse_16bpp_packed(
 	u_short icol;
 
 	icol = tfbm_select_16_color(color);
-	for (y = sy ; y < sy+ly ; y++) {
+	for (y = sy ; y < min(sy+ly, p->height) ; y++) {
 		d = (u_short*)(p->smem + y * p->bytePerLine + sx * 2);
 		for (x = 0 ; x < lx ; x++) {
 			d[x] ^= icol;
