@@ -401,10 +401,14 @@ static void tfont_setup_font(TFont* pf, const char* fname, FONTSET_HALF hf)
 
 	if ((nf > 3 && strcmp(".gz", fname+nf-3) == 0) ||
 	    (nf > 2 && strcmp(".Z", fname+nf-2) == 0)) {
+		if (strlen(JFB_GZIP_PATH) + 10 + nf >= sizeof cmd) {
+			printf("PCF : long filename? : %s\n", fname);
+			return;
+		}
 #ifndef JFB_GZIP_PATH
 #error		JFB_GZIP_PATH is not set.
 #else
-		strcpy(cmd, JFB_GZIP_PATH " -dc < \"");
+		strcpy(cmd, JFB_GZIP_PATH " -c < \"");
 #endif
 		strcat(cmd, fname);
 		strcat(cmd, "\"");
