@@ -148,7 +148,9 @@ void tapp_init(TApplication* p)
 	p->gCapsQ	= TRUE;
 	p->gOptShowHelpQ = FALSE;
 	if (getenv("SHELL")) {
-		strcpy(shell, getenv("SHELL"));
+		/* This cause a buffer overflow. */
+		memset (shell, '\0', sizeof shell);
+		strncpy(shell, getenv("SHELL"), sizeof shell - 1);
 		p->gExecShell = shell;
 	} else {
 		p->gExecShell = "/bin/sh";
