@@ -32,6 +32,7 @@
 #include <errno.h>
 #include <unistd.h>
 #include <dlfcn.h>
+#include <time.h>
 #include <sys/types.h>
 #include <sys/wait.h>
 #include <sys/time.h>
@@ -49,6 +50,7 @@
 #include <message.h>
 #include <main.h>
 
+#include "config.h"
 
 int gChildProcessId = 0;
 
@@ -151,16 +153,18 @@ void tterm_start(TTerm* p, const char* tn, const char* en)
 	int ret;
 	struct timeval tv;
 	u_char buf[BUF_SIZE+1];
+#ifdef JFB_ENABLE_DIMMER
+	u_int idle_time = 0;
+	u_int blank = 0;
+	int tfbm_set_blank(int, int);
+#  define DIMMER_TIMEOUT (3 * 60 * 10)        /* 3 min */
+#endif
 
 	tterm_init(p, en);
 	if (!tterm_get_ptytty(p)) {
 		die("Cannot get free pty-tty.\n");
 	}
 
-	tterm_wakeup_shell_as_child(p, tn);
-
-	atexit(application_final);
-
 	ntio = p->ttysave;
 	ntio.c_lflag &= ~(ECHO|ISIG|ICANON|XCASE);
         ntio.c_iflag = 0;
@@ -175,6 +179,10 @@ void tterm_start(TTerm* p, const char* tn, const char* en)
 */
 
 	tvterm_start(&(p->vterm));
+	tterm_wakeup_shell_as_child(p, tn);
+
+	atexit(application_final);
+
 	/* not available
 	 * VtInit();
 	 * VtStart();
@@ -183,13 +191,21 @@ void tterm_start(TTerm* p, const char* tn, const char* en)
 		fd_set fds;
 		int max = 0;
 		tv.tv_sec = 0;
-		tv.tv_usec = 100000;
+		tv.tv_usec = 100000;	// 100 msec
 		FD_ZERO(&fds);
 		FD_SET(0,&fds);
 		FD_SET(p->ptyfd,&fds);
 		if (p->ptyfd > max) max = p->ptyfd;
 		ret = select(max+1, &fds, NULL, NULL, &tv);
                 if (ret == 0 || (ret < 0 && errno == EINTR)) {
+#ifdef JFB_ENABLE_DIMMER
+			if (!blank && ++idle_time > DIMMER_TIMEOUT) {
+				// Goto blank
+				idle_time = 0;
+				if (tfbm_set_blank(gFramebuffer.fh, 1))
+					blank = 1;
+			}
+#endif
 			continue;
 		}
 
@@ -198,6 +214,14 @@ void tterm_start(TTerm* p, const char* tn, const char* en)
 		}
 		if (FD_ISSET(0, &fds)) {
 			ret = read(0, buf, BUF_SIZE);
+#ifdef JFB_ENABLE_DIMMER
+			idle_time = 0;
+			if (blank) {
+				// Wakeup console
+				blank = 0;
+				tfbm_set_blank(gFramebuffer.fh, 0);
+			}
+#endif
 			if (ret > 0) {
 				write(p->ptyfd, buf, ret);
 			}
