@@ -45,6 +45,7 @@
 #include	<term.h>
 #include	<font.h>
 #include	<fbcommon.h>
+#include	<config.h>
 
 
 #if 0
@@ -334,11 +335,35 @@ void tvterm_show_cursor(TVterm* p, TBool b)
 		return;
 	}
 	if (p->cursor.shown != b) {
+#ifdef JFB_REVERSEVIDEO
+/* -- kei --
+	On this implementation, JFB_REVERSEVIDEO is effective on
+	only 2bpp machine.
+	On reverse video mode, cursor should be 0x00.
+	On non reverse video mode, cursor should be 0xf.
+	Yes, I know following is little bit dirty code, but
+	temporal fix...
+*/
+		if (gFramebuffer.cap.bitsPerPixel == 2) {
+			gFramebuffer.cap.reverse(&gFramebuffer,
+				gFontsWidth * p->cursor.x,
+				gFontsHeight * p->cursor.y,
+				p->cursor.width + (p->cursor.wide ? p->cursor.width : 0),
+				p->cursor.height, 0x0);
+		} else {
+			gFramebuffer.cap.reverse(&gFramebuffer,
+				gFontsWidth * p->cursor.x,
+				gFontsHeight * p->cursor.y,
+				p->cursor.width + (p->cursor.wide ? p->cursor.width : 0),
+				p->cursor.height, 0xf);
+		}
+#else
 		gFramebuffer.cap.reverse(&gFramebuffer,
 			gFontsWidth * p->cursor.x,
 			gFontsHeight * p->cursor.y,
 			p->cursor.width + (p->cursor.wide ? p->cursor.width : 0),
 			p->cursor.height, 0xf);
+#endif
 		p->cursor.shown = b;
 	}
 }
@@ -369,8 +394,8 @@ void tvterm_refresh(TVterm* p)
 		p->textClear = FALSE;
 	}
 
-	for (x = 0; x < p->xcap; x ++) {
-		for (y = 0; y < p->ycap; y ++) {
+	for (y = 0; y < p->ycap; y ++) {
+		for (x = 0; x < p->xcap; x ++) {
 			i = tvterm_coord_to_index(p, x, y);
 			fg = p->flag[i];
 			if (fg & CLEAN_S) {
@@ -439,13 +464,11 @@ void tvterm_unregister_signal(void)
         vtm.acqsig = 0;
         ioctl(0, VT_SETMODE, &vtm);
 
-        ioctl(sig_obj->term->ttyfd, TIOCSWINSZ, sig_obj->win);
         ioctl(0, TIOCCONS, NULL);
 }
 
 void tvterm_register_signal(TVterm* p)
 {
-        struct winsize win;
         struct vt_mode vtm;
 
         sig_obj = p;
@@ -459,10 +482,6 @@ void tvterm_register_signal(TVterm* p)
         vtm.acqsig = SIGUSR2;
         ioctl(0, VT_SETMODE, &vtm);
 
-        win.ws_row = p->ycap;
-        win.ws_col = p->ycap;
-        win.ws_xpixel = win.ws_ypixel = 0;
-        ioctl(sig_obj->term->ttyfd, TIOCSWINSZ, sig_obj->win);
         ioctl(sig_obj->term->ttyfd, TIOCCONS, NULL);
 
         llatch(p->flag, p->tsize);
@@ -568,7 +587,7 @@ mode
 1	 [0, TVterm::y) 򥯥ꥢ
 	 TVterm::y [0, TVterm::x) 򥯥ꥢ롣
 2	꡼Τ򥯥ꥢ롣
-ow	 [TVterm::y, TVterm::ycap) 򥯥ꥢ
+ow	 [TVterm::y+1, TVterm::ycap) 򥯥ꥢ
 	 TVterm::y [TVterm::x, TVterm::xcap) 򥯥ꥢ롣
 **/
 void tvterm_text_clear_eos(TVterm* p, u_char mode)
@@ -586,7 +605,7 @@ void tvterm_text_clear_eos(TVterm* p, u_char mode)
 		tvterm_text_clear_all(p);
 		break;
 	default:
-		tvterm_text_clean_band(p, p->pen.y, p->ycap);
+		tvterm_text_clean_band(p, p->pen.y + 1, p->ycap);
 		a = tvterm_coord_to_index(p, p->pen.x, p->pen.y);
 		len = p->xcap - p->pen.x;
 		tvterm_bzero2(p, a, len);
